/* eslint class-methods-use-this: 0 */
const _ = require('lodash');
const { Op, QueryTypes } = require('sequelize');
const knex = require('knex')({
    client: 'pg',
    asyncStackTraces: true,
    debug: true
});

const models = require('../models');

let Audit = {};
let sequelize = {};

class AuditDal {

    async init(needCreateDb) {
        if (needCreateDb) {
            await models.createDatabaseIfNotExists();
        }

        ({ Audit, sequelize } = models.getInstance());
    }

    store(audit) {
        return Audit.create(audit);
    }

    _appendWhereCondition(selectQuery, conditions) {
        selectQuery.where((builder) => {
            Object.keys(conditions).forEach((key) => {
                const operator = (_.isObject(conditions[key])) ? '!=' : '=';
                builder = builder.andWhere(key, operator, `${conditions[key]}`);
            });
        });
    }

    async getAudit(query) {
        const { where } = query;
        if (where.createdAt || where[Op.or]) {
            return Audit.findAndCountAll(query);
        }

        const basicQuery = knex({ Audit: 'public.Audits' }).select();

        this._appendWhereCondition(basicQuery, query.where);

        const fromQuery = basicQuery.limit(100000);

        const count = knex({ c: fromQuery })
            .count('* as int')
            .toString();

        const amount = await sequelize.query(`(${count})`, { type: QueryTypes.SELECT });

        const result = await Audit.findAll(query);
        return {
            rows: result,
            count: Number(amount[0].int)
        };
    }

    getAuditWithoutCount(query) {
        return Audit.findAll(query);
    }

    async getUserNames(accountId) {
        const data = await Audit.aggregate('userName', 'DISTINCT', {
            plain: false,
            where: { accountId },
        });
        return data.map(item => item.DISTINCT);
    }

    getEntitiesList(accountId, opts) {
        const where = { accountId };

        if (_.get(opts, 'showTriggers')) {
            where.action = 'receiveWebhook';
        } else {
            where.action = { [Op.ne]: 'receiveWebhook' };
        }

        return Audit.findAll({
            where,
            attributes: ['entity', 'action'],
            distinct: true,
            raw: true,
        });
    }
}

module.exports = new AuditDal();
